# -*- coding: utf-8 -*-

import os
import stat
from uuid import uuid1
from subprocess import Popen
from sys import stdin, stdout, stderr

from tessera import Tessera
from gittessera import GitTessera
from gittle import Gittle
import shutil


class GitCommands(object):

    def __init__(self):
        self.gitdir = "."
        self.git = Gittle(self.gitdir)
        Tessera._tesserae = os.path.relpath("%s/.tesserae" % self.gitdir)

    def cmd_init(self, args):
        if len(args) != 0:
            stderr.write("git tessera init takes no arguments\n")
            return False

        #if self.git.is_dirty():
            #stderr.write("repo is dirty\n")
            #return False

        if os.path.exists(Tessera._tesserae):
            stderr.write("git tesserae directory already exists: %s\n" % Tessera._tesserae)
            return False
        os.mkdir(Tessera._tesserae)

        files = []
        t = "%s/template" % Tessera._tesserae
        shutil.copyfile("%s/config/template" % os.path.dirname(os.path.realpath(__file__)), t)
        files.append(t)

        t = "%s/status" % Tessera._tesserae
        shutil.copyfile("%s/config/status" % os.path.dirname(os.path.realpath(__file__)), t)
        files.append(t)

        t = "%s/types" % Tessera._tesserae
        shutil.copyfile("%s/config/types" % os.path.dirname(os.path.realpath(__file__)), t)
        files.append(t)

        self.git_add(files, "tessera: initialized")
        return True

    def cmd_ls(self, args):
        gt = GitTessera()
        tesserae = gt.ls(args)
        for t in tesserae:
            print t.summary()
        return True

    def cmd_show(self, args):
        if len(args) != 1:
            stderr.write("git tessera show takes identifier as argument\n")
            return False

        gt = GitTessera()
        t = gt.get(args[0])
        if not t:
            return False

        short = t.summary()
        length = len(short)
        print "=" * length
        print short
        print "=" * length
        print t.content
        return True

    def cmd_edit(self, args):
        if len(args) < 1:
            stderr.write("git tessera edit takes one or more identifier as argument\n")
            return False

        tessera_paths = []
        for key in args:
            tessera_path = None
            found = False
            for i in os.listdir(Tessera._tesserae):
                tessera_path = "%s/%s" % (Tessera._tesserae, i)
                if not stat.S_ISDIR(os.lstat(tessera_path).st_mode):
                    continue
                if i.split('-')[0] == key or i == key:
                    found = True
                    break
            if not found:
                stderr.write("git tessera %s not found\n" % key)
                return False

            tessera_paths.append(tessera_path)

        tessera_files = ["%s/tessera" % x for x in tessera_paths]
        p = Popen(["sensible-editor"] + tessera_files)
        p.communicate()
        p.wait()

        #if self.git.is_dirty():
        for tessera_path in tessera_paths:
            t = Tessera(tessera_path)
            self.git_add("%s/tessera" % tessera_path, "tessera updated: %s" % t.title)
        return True

    def cmd_create(self, args):
        if len(args) < 1:
            stderr.write("git tessera create needs arguments\n")
            return False

        #if self.git.is_dirty():
        #    stderr.write("repo is dirty\n")
        #    return False

        if args:
            title = " ".join(args)
        else:
            title = "tessera title goes here"
        uuid = uuid1()
        tessera_path = "%s/%s" % (Tessera._tesserae, uuid)
        tessera_file = "%s/tessera" % tessera_path
        os.mkdir(tessera_path)
        fin = open("%s/template" % Tessera._tesserae, "r")
        fout = open(tessera_file, "w")
        for line in fin.readlines():
            if line == "@title@\n":
                line = "# %s\n" % title
            fout.write(line)
        fin.close()
        fout.close()

        p = Popen(["sensible-editor", tessera_file])
        p.communicate()
        p.wait()

        t = Tessera(tessera_path)
        self.git_add(tessera_file, "tessera created: %s" % t.get_attribute("title"))
        return True

    def cmd_remove(self, args):
        if len(args) != 1:
            stderr.write("git tessera remove takes identifier as argument\n")
            return False

        #if self.git.is_dirty():
            #stderr.write("repo is dirty\n")
            #return False

        key = args[0]
        tessera_file = None
        tessera_path = None
        for i in os.listdir(Tessera._tesserae):
            tessera_path = "%s/%s" % (Tessera._tesserae, i)
            if not stat.S_ISDIR(os.lstat(tessera_path).st_mode):
                continue
            if i.split('-')[0] == key or i == key:
                tessera_file = "%s/tessera" % tessera_path
                break
        if not tessera_file:
            stderr.write("git tessera %s not found\n" % key)
            return False

        t = Tessera(tessera_path)
        stdout.write("remove tessera %s: %s ? [Y/n] " % (key, t.get_attribute("title")))
        try:
            answer = stdin.readline().strip()
        except KeyboardInterrupt:
            return False
        if not answer or answer.lower() == "y":
            files = ["%s/%s" % (tessera_path, x) for x in os.listdir(tessera_path)]
            self.git_rm(files, "tessera removed: %s" % t.get_attribute("title"))

            from shutil import rmtree
            rmtree(tessera_path)

    def cmd_serve(self, args):
        from tesseraweb import TesseraWeb
        web = TesseraWeb()
        web.serve()

    def cmd_tag(self, args):
        if len(args) != 2:
            stderr.write("git tessera show takes identifier as argument and new tag\n")
            return False

        key = args[0]
        for i in os.listdir(Tessera._tesserae):
            tessera_path = "%s/%s" % (Tessera._tesserae, i)
            if not stat.S_ISDIR(os.lstat(tessera_path).st_mode):
                continue
            if i.split('-')[0] == key or i == key:
                break
        if not tessera_path:
            stderr.write("git tessera %s not found\n" % key)
            return False

        t = Tessera(tessera_path)
        t.add_tag(args[1])
        self.git_add(t.filename, "tessera updated: add tag %s to %s" % (args[1], t.get_attribute("title")))
        return True

    def git_add(self, files, message):
        stderr.write("staging %s" % files)
        self.git.commit(message=message, files=files)

    def git_rm(self, files, message):
        self.git.rm(files)
        self.git.commit(message=message)



# -*- coding: utf-8 -*-

import web
import sys
from os import path
from gittessera import GitTessera
import markdown

render = web.template.render('%s/web' % path.dirname(path.realpath(__file__)))


class TesseraWeb(object):
    def __init__(self):
        self.urls = ('/', 'index',
                     '/style/tessera.css', 'css',
                     '/tessera(.*)', 'tessera'
                    )

    def serve(self):
        sys.argv = []
        app = web.application(self.urls, globals())
        app.run()


class index(object):
    def GET(self):
        gt = GitTessera()
        tesserae = gt.ls()
        return render.index(tesserae)


class css:
    def GET(self):
        f = open('%s/web/style/tessera.css' % path.dirname(path.realpath(__file__)), "r")
        stylesheet = f.read()
        f.close()
        return stylesheet


class tessera:
    def GET(self, key):
        i = web.input(key=None)
        gt = GitTessera()
        tessera = gt.get(i.key)
        if not tessera:
            return "not found"
        render = web.template.render('%s/web' % path.dirname(path.realpath(__file__)))
        tessera.markdown = markdown.markdown(tessera.content)
        return render.detail(tessera)



# -*- coding: utf-8 -*-

import os
import re

from sys import stderr

from colorful import colorful


class Tessera(object):
    _tesserae = None
    _status = {}
    _te_types = {}

    def __init__(self, tessera_path):
        self.tessera_path = tessera_path
        self.filename = "%s/tessera" % tessera_path
        self.mtime = os.lstat(self.filename).st_mtime
        self.tessera_hash = os.path.basename(self.tessera_path)
        self._attributes = {}

        if not self._status:
            self._read_status()

        if not self._te_types:
            self._read_types()

        self._read()
        self._parse()

    def get_attribute(self, attribute):
        return self._attributes.get(attribute, "no %s" % attribute)

    def _read_status(self):
        status_file = "%s/status" % Tessera._tesserae
        Tessera._status = self._read_config(status_file)

    def _read_types(self):
        types_file = "%s/types" % Tessera._tesserae
        Tessera._te_types = self._read_config(types_file)

    def _read_config(self, filename):
        if not os.path.exists(filename):
            colorful.out.bold_red("file not found: %s" % filename)
            Tessera._te_types = False
            return

        config = {}
        with open(filename, 'r') as f:
            i = 0
            for line in f.readlines():
                line = line.strip()
                if line:
                    a = re.split(r'[ \t]+', line)
                    if len(a) != 2:
                        colorful.out.bold_red("Error in %s: invalid status line: %s" % (filename, line))
                        break
                    if not colorful.exists(a[1]):
                        colorful.out.bold_red("Error in %s: color %s does not exist" % (filename, a[1]))
                        break
                    config[i] = (a[0], a[1])
                    i += 1
        return config

    def _read(self):
        if not os.path.exists(self.filename):
            stderr.write("tessera file not found: %s\n" % self.fielname)
            return None

        f = open(self.filename, 'r')
        self.body = f.read().split('\n')
        f.close()

    def _parse(self):
        self._attributes["tags"] = set()
        self.content = []
        title_pattern = re.compile("^# (?P<title>.*)$")
        status_pattern = re.compile("^@status (?P<status>.*)$")
        type_pattern = re.compile("^@type (?P<type>.*)$")
        tags_pattern = re.compile("^@tags (?P<tags>.*)$")
        in_header = True
        for l in self.body:
            if in_header:
                if not l.strip():
                    continue

                title_match = title_pattern.search(l)
                if title_match:
                    self._attributes["title"] = title_match.groupdict()["title"]
                    continue
                status_match = status_pattern.search(l)
                if status_match:
                    self._attributes["status"] = status_match.groupdict()["status"]
                    continue
                type_match = type_pattern.search(l)
                if type_match:
                    self._attributes["type"] = type_match.groupdict()["type"]
                    continue
                tags_match = tags_pattern.search(l)
                if tags_match:
                    self._attributes["tags"] = set([x.strip() for x in tags_match.groupdict()["tags"].split(",")])
                    continue

                in_header = False
            self.content.append(l)
        self.content = "\n".join(self.content)

        self.status_id = -1
        if self._attributes["status"]:
            for key, val in Tessera._status.iteritems():
                if val[0] == self._attributes["status"]:
                    self.status_id = key
                    break

        self.te_type_id = -1
        if self._attributes["type"]:
            for key, val in Tessera._te_types.iteritems():
                if val[0] == self._attributes["type"]:
                    self.te_type_id = key
                    break

    def _write(self):
        with open(self.filename, "w") as f:
            f.write("# %s\n" % self.get_attribute("title"))
            f.write("@status %s\n" % self.get_attribute("status"))
            f.write("@type %s\n" % self.get_attribute("type"))
            f.write("@tags %s\n" % ", ".join(self._attributes["tags"]))
            f.write("\n%s" % self.content)

    def summary(self):
        title = self.get_attribute("title")
        status = self.get_attribute("status")
        te_type = self.get_attribute("type")
        len_title = len(title)
        len_status = len(status)
        tags = ", ".join(self._attributes["tags"])
        color = None
        if Tessera._status:
            for s in Tessera._status:
                if Tessera._status[s][0] == status:
                    color = Tessera._status[s][1]
                    break
        else:
            status = "no status available"
        if color:
            if colorful.exists(color):
                status = colorful.get(color)(status)

        color = None
        if Tessera._te_types:
            for s in Tessera._te_types:
                if Tessera._te_types[s][0] == te_type:
                    color = Tessera._te_types[s][1]
                    break
        else:
            te_type = "no type available"
        if color:
            if colorful.exists(color):
                title = colorful.get(color)(title)

        return "%s %s %s %s %s %s %s %s" % (self.get_ident_short(), title, " " * (40 - len_title), status, " " * (10 - len_status), te_type, " " * (10 - len(te_type)), tags)

    def add_tag(self, tag):
        self._attributes["tags"].add(tag)
        self._write()
        return True

    def ident(self):
        return dict(ident=self.tessera_hash, title=self.get_attribute("title"), filename=self.filename, body=self.get_body())

    def get_ident_short(self):
        return self.tessera_hash.split('-')[0]




#!/usr/bin/env python
# -*- coding: utf-8 -*-

from sys import argv, stderr, exit
from gitcommands import GitCommands


def main():
    cmd = "ls"
    if len(argv) > 1:
        cmd = argv[1]

    #try:
    t = GitCommands()
    #except git.exc.InvalidGitRepositoryError:
        #stderr.write("not a git repo\n")
        #exit(1)
    if hasattr(t, "cmd_%s" % cmd):
        if not getattr(t, "cmd_%s" % cmd)(argv[2:]):
            exit(1)
        exit(0)
    else:
        stderr.write("unknown command: %s\n" % cmd)
        exit(2)

if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-

import os
import stat
from sys import stderr

from gittle import Gittle
from tessera import Tessera
from colorful import colorful


class GitTessera(object):
    SORTING = {
        "date": lambda t1, t2: t1.mtime < t2.mtime,
        "status": lambda t1, t2: cmp(t1.status_id, t2.status_id),
        "title": lambda t1, t2: cmp(t1.get_attribute("title").lower(), t2.get_attribute("title").lower()),
        "hash": lambda t1, t2: cmp(t1.tessera_hash.lower(), t2.tessera_hash.lower())
    }

    def __init__(self):
        self.gitdir = "."
        self.git = Gittle(self.gitdir)
        self.tesserae = "%s/.tesserae" % self.gitdir

    def ls(self, args=[]):
        if not os.path.exists(self.tesserae):
            return False

        try:
            idx = args.index("--sort")
            sortfunc = GitTessera.SORTING[args[idx + 1]]
        except ValueError:
            sortfunc = GitTessera.SORTING["status"]
        except IndexError:
            colorful.out.bold_red("Please specify a sort algorithm. Available: %s" % (", ".join(GitTessera.SORTING.keys())))
            return []
        except KeyError:
            colorful.out.bold_red("No sort algorithm for '%s' available" % args[idx + 1])
            return []

        try:
            idx = args.index("--tags")
            tags = args[idx + 1].split(",")
        except ValueError:
            tags = None
        except IndexError:
            colorful.out.bold_red("Please specify minimum one tag.")
            return []

        contents = [self.tesserae + "/" + x for x in os.listdir(self.tesserae) if stat.S_ISDIR(os.lstat(self.tesserae + "/" + x).st_mode)]
        tesserae = []
        for tessera_path in contents:
            t = Tessera(tessera_path)
            te_tags = t.get_attribute("tags")
            if not tags or any(x in te_tags for x in tags):
                tesserae.append(t)
        tesserae = sorted(tesserae, cmp=sortfunc)
        return tesserae

    def get(self, key):
        for i in os.listdir(self.tesserae):
            tessera_path = "%s/%s" % (self.tesserae, i)
            if not stat.S_ISDIR(os.lstat(tessera_path).st_mode):
                continue
            if i.split('-')[0] == key or i == key:
                break
        if not tessera_path:
            stderr.write("git tessera %s not found\n" % key)
            return None
        return Tessera(tessera_path)



#!/usr/bin/python
# -*- coding: utf-8 -*-

from imp import load_source
from setuptools import setup

tessera_version = load_source("version", "tessera/version.py")

setup(
    name="git-tessera",
    version=tessera_version.__version__,
    license="GPL",
    description="manage your issues in your repository",
    author="André Roth, Claudio Klingler",
    author_email="neolynx@gmail.com",
    maintainer="André Roth",
    maintainer_email="neolynx@gmail.com",
    platforms=["Linux", "Windows", "MAC OS X"],
    url="https://github.com/neolynx/git-tessera.git",
    download_url="https://github.com/neolynx/git-tessera.git",
    install_requires=["gittle==0.2.2", "web.py", "colorful==0.01.02", "markdown"],
    packages=["tessera"],
    entry_points={"console_scripts": ["git-tessera = tessera.main:main"]},
    package_dir={'git-tessera': 'tessera'},
    package_data={ "git-tessera": ["config/*", "web/*", "web/style/*"]},
    include_package_data=True,
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Intended Audience :: Education",
        "Intended Audience :: Other Audience",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Natural Language :: English",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: OS Independent",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: Microsoft :: Windows :: Windows 7",
        "Operating System :: Microsoft :: Windows :: Windows XP",
        "Operating System :: POSIX",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.2",
        "Programming Language :: Python :: Implementation",
        "Topic :: Education :: Testing",
        "Topic :: Software Development",
        "Topic :: Software Development :: Testing"
    ],
)
